package com.getmek.odb.odb_connect;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;

import androidx.annotation.NonNull;

import com.sohrab.obd.reader.application.Preferences;
import com.sohrab.obd.reader.obdCommand.ObdCommand;
import com.sohrab.obd.reader.obdCommand.ObdConfiguration;
import com.sohrab.obd.reader.obdCommand.SpeedCommand;
import com.sohrab.obd.reader.obdCommand.control.DistanceMILOnCommand;
import com.sohrab.obd.reader.obdCommand.control.ModuleVoltageCommand;
import com.sohrab.obd.reader.obdCommand.control.PendingTroubleCodesCommand;
import com.sohrab.obd.reader.obdCommand.control.PermanentTroubleCodesCommand;
import com.sohrab.obd.reader.obdCommand.control.TimingAdvanceCommand;
import com.sohrab.obd.reader.obdCommand.control.TroubleCodesCommand;
import com.sohrab.obd.reader.obdCommand.control.VinCommand;
import com.sohrab.obd.reader.obdCommand.engine.LoadCommand;
import com.sohrab.obd.reader.obdCommand.engine.MassAirFlowCommand;
import com.sohrab.obd.reader.obdCommand.engine.OilTempCommand;
import com.sohrab.obd.reader.obdCommand.engine.RPMCommand;
import com.sohrab.obd.reader.obdCommand.engine.RuntimeCommand;
import com.sohrab.obd.reader.obdCommand.engine.ThrottlePositionCommand;
import com.sohrab.obd.reader.obdCommand.fuel.AirFuelRatioCommand;
import com.sohrab.obd.reader.obdCommand.pressure.FuelPressureCommand;
import com.sohrab.obd.reader.obdCommand.temperature.AirIntakeTemperatureCommand;
import com.sohrab.obd.reader.obdCommand.temperature.EngineCoolantTemperatureCommand;
import com.sohrab.obd.reader.service.ObdReaderService;
import com.sohrab.obd.reader.trip.TripRecord;

import java.util.ArrayList;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import static com.sohrab.obd.reader.constants.DefineObdReader.ACTION_CONNECTION_STATUS_MSG;
import static com.sohrab.obd.reader.constants.DefineObdReader.ACTION_READ_OBD_REAL_TIME_DATA;

/**
 * OdbConnectPlugin
 */
public class OdbConnectPlugin implements FlutterPlugin, MethodCallHandler {
    String obdSpeed;
    String obdEngineCoolantTemp;
    String obdEngineLoad;
    String obdEngineRpm;
    String obdModuleVoltage;
    String obdDistanceMILOn;
    String obdTroubleCodes;
    String obdAirFuelRatio;
    String obdFuelPressure;
    String obdPermanentTroubleCodes;
    String obdViNumber;
    String obdEngineOilTemp;
    String obdEngineRuntime;
    String obdTimingAdvance;

    BroadcastReceiver mObdReaderReceiver;
    Intent obdIntent;
    String obdAction;
    String obdMesg;
    private MethodChannel channel;
    private Context context;

    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "obd_connect");
        channel.setMethodCallHandler(this);
        context = flutterPluginBinding.getApplicationContext();
        float gasPrice = 129.9f; // per litre, you should initialize according to your requirement.
        Preferences.get(context).setGasPrice(gasPrice);
    }


    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        switch (call.method) {
            case "getPlatformVersion":
                result.success("Android " + android.os.Build.VERSION.RELEASE);
                break;
            case "calculate":
                int a = call.argument("a");
                int b = call.argument("b");
                int r = a + b;
                result.success("" + r);
                break;
            case "startOBD":
                ArrayList<ObdCommand> obdCommands = new ArrayList<>();
                obdCommands.add(new VinCommand());
                obdCommands.add(new AirFuelRatioCommand());//air fuel ratio

                obdCommands.add(new FuelPressureCommand());
                obdCommands.add(new ModuleVoltageCommand()); //fuel pressure

                obdCommands.add(new AirIntakeTemperatureCommand()); //Air Intake pressure

                obdCommands.add(new TroubleCodesCommand()); //Trouble codes

                obdCommands.add(new PermanentTroubleCodesCommand()); //Permanent Trouble codes

                obdCommands.add(new PendingTroubleCodesCommand()); //Pending Trouble codes

                //Engine commands
                obdCommands.add(new MassAirFlowCommand()); //MAF

                obdCommands.add(new DistanceMILOnCommand()); //MIL distance

                obdCommands.add(new OilTempCommand()); //Oil Temperature

                obdCommands.add(new RuntimeCommand()); //Engine runtime since ignation
                obdCommands.add(new ThrottlePositionCommand()); //MAF

                obdCommands.add(new SpeedCommand()); //speed

                obdCommands.add(new RPMCommand()); //rpm

                obdCommands.add(new LoadCommand()); //engine load

                obdCommands.add(new EngineCoolantTemperatureCommand()); //coolant temp

                obdCommands.add(new TimingAdvanceCommand()); //Timing Level

                ObdConfiguration.setmObdCommands(context, obdCommands);

                //Register receiver with some action related to OBD connection status and read PID values
                IntentFilter intentFilter = new IntentFilter();
                intentFilter.addAction(ACTION_READ_OBD_REAL_TIME_DATA);
                intentFilter.addAction(ACTION_CONNECTION_STATUS_MSG);

                //bluetooth receiver method
                mObdReaderReceiver = new BroadcastReceiver() {
                    @Override
                    public void onReceive(Context context, Intent intent) {
                        obdIntent = intent;
                        obdAction = intent.getAction();
                        // System.out.println("Hello, World 2");
                        if (obdAction.equals(ACTION_CONNECTION_STATUS_MSG)) {
                            String connectionStatusMsg = intent.getStringExtra(ObdReaderService.INTENT_EXTRA_DATA);
                            //OBD connected  do what want after OBD connection
                            // System.out.println("Hello, World 3");
                            System.out.println(context.getString(R.string.connected_ok));
                            if (connectionStatusMsg.equals(context.getString(R.string.connected_ok))) {
                                //OBD connected  do what want after OBD connection
                                // System.out.println("蓝牙连接成功");
                                obdMesg = "ODB connected successfully";

                            } else {
                                //OBD connected  do what want after OBD connection
                                // System.out.println("蓝牙连接不成功");
                                obdMesg = "ODB Not connected";
                            }
                        }
                    }
                };
                // System.out.println("Hello, World 1");
                context.registerReceiver(mObdReaderReceiver, intentFilter);
                //start service that keep running in background for connecting and execute command until you stop
                context.startService(new Intent(context, ObdReaderService.class));
                result.success("Service running");
                break;
            case "getTripRecord":
                System.out.println("getTripRecord");
                if (obdAction.equals(ACTION_CONNECTION_STATUS_MSG)) {
                    String connectionStatusMsg = obdIntent.getStringExtra(ObdReaderService.INTENT_EXTRA_DATA);
                    //OBD connected  do what want after OBD connection
                    // System.out.println("Hello, World 3");
                    System.out.println(context.getString(R.string.connected_ok));
                    if (connectionStatusMsg.equals(context.getString(R.string.connected_ok))) {
                        //OBD connected  do what want after OBD connection
                        System.out.println("蓝牙连接成功");
                    } else {
                        //OBD connected  do what want after OBD connection
                        System.out.println("蓝牙连接不成功");
                    }
                } else if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getmEngineCoolantTemp() != null) {
                        obdEngineCoolantTemp = String.valueOf(tripRecord.getmEngineCoolantTemp());//display coolant temp
                    } else {
                        obdEngineCoolantTemp = "no EngineCoolantTemp signal";
                    }
                }
                result.success("" + obdEngineCoolantTemp);
                break;
            case "getSpeed":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getSpeed() != null) {
                        obdSpeed = String.valueOf(tripRecord.getSpeed()); //display speed
                    } else {
                        obdSpeed = "no Speed signal";
                    }
                }
                result.success("" + obdSpeed);
                break;
            case "getRPM":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getEngineRpm() != null) {
                        obdEngineRpm = String.valueOf(tripRecord.getEngineRpm()); //display speed
                    } else {
                        obdEngineRpm = "no Speed signal";
                    }
                }
                result.success("" + obdEngineRpm);
                break;
            case "getAirIntakeTemperature":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getmEngineCoolantTemp() != null) {
                        obdEngineCoolantTemp = String.valueOf(tripRecord.getmEngineCoolantTemp());//display coolant temp
                    } else {
                        obdEngineCoolantTemp = "no EngineCoolantTemp signal";
                    }
                }
                result.success("" + obdEngineCoolantTemp);

                break;
            case "getEngineLoad":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getmEngineLoad() != null) {
                        obdEngineLoad = String.valueOf(tripRecord.getmEngineLoad()); //display speed
                    } else {
                        obdEngineLoad = "no Speed signal";
                    }
                }
                result.success("" + obdEngineLoad);

                break;
            case "getModuleVoltage":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getmControlModuleVoltage() != null) {
                        obdModuleVoltage = String.valueOf(tripRecord.getmControlModuleVoltage()); //display speed
                    } else {
                        obdModuleVoltage = "no Module Voltage signal";
                    }
                }
                result.success("" + obdModuleVoltage);

                break;
            case "getDistanceMILOn":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getmDistanceTraveledMilOn() != null) {
                        obdDistanceMILOn = String.valueOf(tripRecord.getmDistanceTraveledMilOn()); //display speed
                    } else {
                        obdDistanceMILOn = "no Distance MILOn signal";
                    }
                }
                result.success("" + obdDistanceMILOn);
                break;
            case "getTroubleCode":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getmFaultCodes() != null) {
                        obdTroubleCodes = String.valueOf(tripRecord.getmFaultCodes()); //display speed
                    } else {
                        obdTroubleCodes = "no Trouble code";
                    }
                }
                result.success("" + obdTroubleCodes);
                break;
            case "getPermanentTroubleCode":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getmPermanentTroubleCode() != null) {
                        obdPermanentTroubleCodes = String.valueOf(tripRecord.getmPermanentTroubleCode()); //display speed
                    } else {
                        obdPermanentTroubleCodes = "no Permanent Trouble code";
                    }
                }
                result.success("" + obdTroubleCodes);
                break;
            case "getPendingTroubleCode":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getmPendingTroubleCode() != null) {
                        obdPermanentTroubleCodes = String.valueOf(tripRecord.getmPendingTroubleCode()); //display speed
                    } else {
                        obdPermanentTroubleCodes = "no Pending Trouble code";
                    }
                }
                result.success("" + obdTroubleCodes);
                break;
            case "getAirFuelRatio":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getmAirFuelRatio() != null) {
                        obdAirFuelRatio = String.valueOf(tripRecord.getmAirFuelRatio()); //display air/fuel ratio
                    } else {
                        obdAirFuelRatio = "Cannot show air/fuel ratio";
                    }
                }
                result.success("" + obdAirFuelRatio);
                break;
            case "getFuelPressure":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getmFuelPressure() != null) {
                        obdFuelPressure = String.valueOf(tripRecord.getmFuelPressure()); //display fuel pressure
                    } else {
                        obdFuelPressure = "Cannot show fuel pressure";
                    }
                }
                result.success("" + obdFuelPressure);
                break;
            case "getGasCost":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    obdFuelPressure = String.valueOf(tripRecord.getmGasCost()); //display gas cost
                }
                result.success("" + obdFuelPressure);
                break;
            case "getVINumber":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    obdViNumber = String.valueOf(tripRecord.getmVehicleIdentificationNumber()); //display Vehicle ID number
                }
                result.success("" + obdViNumber);
                break;
            case "getOilTemperature":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getmEngineOilTemp() != null) {
                        obdEngineOilTemp = String.valueOf(tripRecord.getmEngineOilTemp()); //display Oil temperature
                    } else {
                        obdEngineOilTemp = "Cannot show Oil temp";
                    }
                }
                result.success("" + obdEngineOilTemp);
                break;
            case "getEngineRuntime":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getEngineRuntime() != null) {
                        obdEngineRuntime = String.valueOf(tripRecord.getEngineRuntime()); //display Engine Runtime
                    } else {
                        obdEngineRuntime = "Cannot show Runtime";
                    }
                }
                result.success("" + obdEngineRuntime);
                break;
            case "getTimingAdvance":
                if (obdAction.equals(ACTION_READ_OBD_REAL_TIME_DATA)) {
                    TripRecord tripRecord = TripRecord.getTripRecode(context);
                    if (tripRecord.getmTimingAdvance() != null) {
                        obdTimingAdvance = String.valueOf(tripRecord.getmTimingAdvance()); //display Timing Advance
                    } else {
                        obdTimingAdvance = "Cannot show Timing";
                    }
                }
                result.success("" + obdTimingAdvance);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }
}
