import Flutter
import UIKit
import OBD2

public class SwiftOdbConnectPlugin: NSObject, FlutterPlugin {
    let obd = OBD2();
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "odb_connect", binaryMessenger: registrar.messenger())
        let instance = SwiftOdbConnectPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {

        if(call.method=="startOBD"){
        obd.connect {(success, error) in
            if let error = error {
                print("OBD connection failed with \(error)")
            } else {
                //perform something
                result("OBD connected " + UIDevice.current.systemVersion)
                
            }
          }
        }
        switch call.method {
        case "getPlatformVersion":
            result(call.method)
            break;
        case "getVINumber":
            obd.request(command: Command.Mode09.vin) { (descriptor) in
                let respStr = descriptor?.VIN()
                result(respStr ?? "No value")
                   }
            break;
        case "getTroubleCode":
            obd.request(command: Command.Mode03.troubleCode) { (descriptor) in
                    let respStr = descriptor?.getTroubleCodes()
                result(respStr ?? "No value")
            }
            break;
        case "getPermanentTroubleCode":
            obd.request(command: Command.Custom.digit(mode: 03, pid: 02)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
             }
            break;
        case "getPendingTroubleCode":
            obd.request(command: Command.Custom.digit(mode: 03, pid: 02)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
             }
            break;
        case "getTripRecord":
            obd.request(command: Command.Custom.digit(mode: 09, pid: 02)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
             }
            break;
        case "getSpeed":
            let command = Command.Mode01.pid(number: 13)
                   if obd.isRepeating(repeat: command) {
                       obd.stop(repeat: command)
                   } else {
                      
                   }
            break;
        case "getRPM":
            obd.request(command: Command.Custom.digit(mode: 09, pid: 02)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
             }
            break;
        case "getAirIntakeTemperature":
            obd.request(command: Command.Custom.digit(mode: 09, pid: 05)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
             }
            break;
        case "getEngineLoad":
            obd.request(command: Command.Custom.digit(mode:01, pid: 04)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
             }
            break;
        case "getModuleVoltage":
            obd.request(command: Command.Custom.digit(mode: 09, pid: 66)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
             }
            break;
        case "getDistanceMILOn":
            obd.request(command: Command.Custom.digit(mode: 09, pid: 33)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
             }
            break;
        case "getAirFuelRatio":
            obd.request(command: Command.Custom.digit(mode: 09, pid: 68)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
             }
            break;
        case "getFuelPressure":
            obd.request(command: Command.Custom.digit(mode: 09, pid: 10)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
             }
            break;
        case "getGasCost":
            obd.request(command: Command.Custom.digit(mode: 09, pid: 02)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
             }
            break;
        case "getOilTemperature":
            obd.request(command: Command.Custom.digit(mode: 90, pid: 92)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
             }
            break;
        case "getEngineRuntime":
            obd.request(command: Command.Custom.digit(mode: 09, pid: 31)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
            }
            break;
        case "getOdometer":
            obd.request(command: Command.Custom.digit(mode: 09, pid: 166)) { (descr) in
                 if let response = descr?.response {
                     result(response)
                 }
            }
            break;
        case "getTimingAdvance":
            obd.request(command: Command.Custom.string("0114")) { (descr) in
                result((String(describing: descr?.getResponse())));
                   }
            break;
        default:
            result("Not implemented");
            break;
        }
    }
}
